@@ -82,8 +82,6 @@ typedef struct {
 
 	GdkPixbuf         *pixbuf;
 
-	GtkWidget         *about_dialog;
-
 	GtkWidget         *preferences_dialog;
 	GtkWidget         *name_entry;
 	GtkWidget         *command_label;
@@ -570,79 +568,6 @@ display_preferences_dialog (GtkAction  *action,
 	g_object_unref (builder);
 }
 
-static void
-display_help_dialog (GtkAction  *action,
-		     FishApplet *fish)
-{
-	show_help (fish, NULL);
-}
-
-static void
-display_about_dialog (GtkAction  *action,
-		      FishApplet *fish)
-{
-	const char *author_format = _("%s the Fish");
-	const char *about_format = _("%s has no use what-so-ever. "
-				     "It only takes up disk space and "
-				     "compilation time, and if loaded it also "
-				     "takes up precious panel space and "
-				     "memory. Anybody found using it should be "
-				     "promptly sent for a psychiatric "
-				     "evaluation.");
-	const char  *documenters [] = {
-		"Telsa Gwynne <hobbit@aloss.ukuu.org.uk>",
-		"Sun GNOME Documentation Team <gdocteam@sun.com>",
-          	NULL
-	};
-
-	char        *authors [3];
-	char        *descr;
-
-	if (fish->about_dialog) {
-		gtk_window_set_screen (GTK_WINDOW (fish->about_dialog),
-				       gtk_widget_get_screen (GTK_WIDGET (fish)));
-		gtk_window_present (GTK_WINDOW (fish->about_dialog));
-		return;
-	}
-
-	authors [0] = g_strdup_printf (author_format, fish->name);
-	authors [1] = _("(with minor help from George)");
-	authors [2] = NULL;
-
-	descr = g_strdup_printf (about_format, fish->name);
-		
-	fish->about_dialog = gtk_about_dialog_new ();
-	g_object_set (fish->about_dialog,
-		      "program-name", _("Fish"),
-		      "version", "3.4.7.4ac19",
-		      "copyright", "Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
-		      "comments", descr,
-		      "authors", (const char **) authors,
-		      "documenters", documenters,
-		      "translator-credits", _("translator-credits"),
-		      "logo-icon-name", FISH_ICON,
-		      NULL);
-
-	g_free (descr);
-	g_free (authors [0]);
-
-	gtk_window_set_icon_name (GTK_WINDOW (fish->about_dialog), FISH_ICON);
-	gtk_window_set_wmclass (
-		GTK_WINDOW (fish->about_dialog), "fish", "Fish");
-	gtk_window_set_screen (GTK_WINDOW (fish->about_dialog),
-			       gtk_widget_get_screen (GTK_WIDGET (fish)));
-
-	g_signal_connect (fish->about_dialog, "destroy",
-			  G_CALLBACK (gtk_widget_destroyed),
-			  &fish->about_dialog);
-
-	g_signal_connect (fish->about_dialog, "response",
-			  G_CALLBACK (gtk_widget_destroy),
-			  NULL);
-
-	gtk_widget_show (fish->about_dialog);
-}
-
 static void
 set_ally_name_desc (GtkWidget  *widget,
 		    FishApplet *fish)
@@ -1476,10 +1401,18 @@ load_fish_image (FishApplet *fish)
 	return TRUE;
 }
 
+static gboolean
+update_surface_in_idle (gpointer data)
+{
+	update_surface (FISH_APPLET (data));
+	return FALSE;
+}
+
 static void
 update_surface (FishApplet *fish)
 {
 	GtkWidget     *widget = fish->drawing_area;
+	GtkRequisition prev_requisition;
 	GtkAllocation  allocation;
 	int            width  = -1;
 	int            height = -1;
@@ -1508,17 +1441,20 @@ update_surface (FishApplet *fish)
 	pixbuf_width  = gdk_pixbuf_get_width  (fish->pixbuf);
 	pixbuf_height = gdk_pixbuf_get_height (fish->pixbuf);
 
+	prev_requisition = fish->requisition;
+
 	if (fish->orientation == PANEL_APPLET_ORIENT_UP ||
 	    fish->orientation == PANEL_APPLET_ORIENT_DOWN) {
 		height = allocation.height;
 		width  = pixbuf_width * ((gdouble) height / pixbuf_height);
+
 		fish->requisition.width = width / fish->n_frames;
 		fish->requisition.height = height;
 	} else {
 		if (!rotate) {
 			width = allocation.width * fish->n_frames;
 			height = pixbuf_height * ((gdouble) width / pixbuf_width);
-			fish->requisition.width = width;
+			fish->requisition.width = allocation.width;
 			fish->requisition.height = height;
 		} else {
 			width = allocation.width;
@@ -1528,9 +1464,12 @@ update_surface (FishApplet *fish)
 		}
 	}
 
-	gtk_widget_set_size_request (fish->drawing_area,
-				     fish->requisition.width,
-				     fish->requisition.height);
+	if (prev_requisition.width  != fish->requisition.width ||
+	    prev_requisition.height != fish->requisition.height) {
+		gtk_widget_set_size_request (widget,
+					     fish->requisition.width,
+					     fish->requisition.height);
+	}
 
 	g_assert (width != -1 && height != -1);
 
@@ -1639,15 +1578,12 @@ fish_applet_size_allocate (GtkWidget     *widget,
 			   GtkAllocation *allocation,
 			   FishApplet    *fish)
 {
-	GtkAllocation widget_allocation;
-
-	gtk_widget_get_allocation (widget, &widget_allocation);
-
-	if (widget_allocation.width  != fish->prev_allocation.width ||
-	    widget_allocation.height != fish->prev_allocation.height)
-		update_surface (fish);
+	if (allocation->width  == fish->prev_allocation.width &&
+	    allocation->height == fish->prev_allocation.height)
+		return;
 
 	fish->prev_allocation = *allocation;
+	g_idle_add (update_surface_in_idle, fish);
 }
 
 static void
@@ -1847,13 +1783,7 @@ setup_fish_widget (FishApplet *fish)
 static const GtkActionEntry fish_menu_verbs [] = {
 	{ "FishPreferences", GTK_STOCK_PROPERTIES, N_("_Preferences"),
 	  NULL, NULL,
-	  G_CALLBACK (display_preferences_dialog) },
-	{ "FishHelp", GTK_STOCK_HELP, N_("_Help"),
-	  NULL, NULL,
-	  G_CALLBACK (display_help_dialog) },
-	{ "FishAbout", GTK_STOCK_ABOUT, N_("_About"),
-	  NULL, NULL,
-	  G_CALLBACK (display_about_dialog) }
+	  G_CALLBACK (display_preferences_dialog) }
 };
 
 static void
@@ -2022,10 +1952,6 @@ fish_applet_dispose (GObject *object)
 		g_object_unref (fish->pixbuf);
 	fish->pixbuf = NULL;
 
-	if (fish->about_dialog)
-		gtk_widget_destroy (fish->about_dialog);
-	fish->about_dialog = NULL;
-
 	if (fish->preferences_dialog)
 		gtk_widget_destroy (fish->preferences_dialog);
 	fish->preferences_dialog = NULL;
@@ -2077,8 +2003,6 @@ fish_applet_instance_init (FishApplet      *fish,
 
 	fish->pixbuf = NULL;
 
-	fish->about_dialog = NULL;
-
 	fish->preferences_dialog = NULL;
 	fish->name_entry         = NULL;
 	fish->command_label      = NULL;
