@@ -3,13 +3,13 @@
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 #include <gtk/gtk.h>
-#include <libgnomeui/gnome-ui-init.h>
+
+#include <libpanel-util/panel-error.h>
+#include <libpanel-util/panel-keyfile.h>
 
 #include "panel-ditem-editor.h"
-#include "panel-util.h"
 #include "panel-icon-names.h"
-
-#include "nothing.cP"
+#include "panel-util.h"
 
 /* FIXME Symbols needed by panel-util.c - sucky */
 #include "applet.h"
@@ -18,12 +18,18 @@ GSList *panel_applet_list_applets (void) { return NULL; }
 GConfClient *panel_gconf_get_client (void) { return NULL; }
 #include "panel-config-global.h"
 gboolean panel_global_config_get_tooltips_enabled (void) { return FALSE; }
+#include "panel-lockdown.h"
+gboolean panel_lockdown_get_disable_lock_screen (void) { return FALSE; }
+#include "panel-bindings.h"
+guint panel_bindings_get_mouse_button_modifier_keymask (void) { return 0; }
+#include "panel-toplevel.h"
+GType panel_toplevel_get_type (void) { return 0; }
 
 static int dialogs = 0;
 static gboolean create_new = FALSE;
 static char **desktops = NULL;
 
-static const GOptionEntry options[] = {
+static GOptionEntry options[] = {
 	{ "create-new", 0, 0, G_OPTION_ARG_NONE, &create_new, N_("Create new file in the given directory"), NULL },
 	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &desktops, NULL, N_("[FILE...]") },
 	{ NULL }
@@ -64,7 +70,7 @@ find_uri_on_save (PanelDItemEditor *dialog,
 	char     *dir;
 
 	keyfile = panel_ditem_editor_get_key_file (dialog);
-	name = panel_util_key_file_get_string (keyfile, "Name");
+	name = panel_key_file_get_string (keyfile, "Name");
 
 	validate_for_filename (name);
 	filename = g_filename_from_utf8 (name, -1, NULL, NULL, NULL);
@@ -95,23 +101,22 @@ error_reported (GtkWidget  *dialog,
 int
 main (int argc, char * argv[])
 {
-	GOptionContext *context;
-	GnomeProgram *program;
+	GError *error = NULL;
 	int i;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	context = g_option_context_new (N_("- Edit .desktop files"));
-	g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
-	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
-
-	program = gnome_program_init ("gnome-desktop-item-edit", VERSION,
-				      LIBGNOMEUI_MODULE,
-				      argc, argv,
-				      GNOME_PARAM_GOPTION_CONTEXT, context,
-				      NULL);
+	if (!gtk_init_with_args (&argc, &argv,
+	                         _("- Edit .desktop files"),
+	                         options,
+	                         GETTEXT_PACKAGE,
+	                         &error)) {
+		g_printerr ("%s\n", error->message);
+		g_error_free (error);
+		return 1;
+	}
 
 	gtk_window_set_default_icon_name (PANEL_ICON_LAUNCHER);
 
@@ -126,10 +131,12 @@ main (int argc, char * argv[])
 		GFileInfo *info;
 		GFileType  type;
 		char      *uri;
+		char      *path;
 		GtkWidget *dlg = NULL;
 
 		file = g_file_new_for_commandline_arg (desktops[i]);
 		uri  = g_file_get_uri (file);
+		path = g_file_get_path (file);
 		info = g_file_query_info (file, "standard::type",
 					  G_FILE_QUERY_INFO_NONE, NULL, NULL);
 		g_object_unref (file);
@@ -138,10 +145,11 @@ main (int argc, char * argv[])
 			type = g_file_info_get_file_type (info);
 
 			if (type == G_FILE_TYPE_DIRECTORY && create_new) {
+
 				dlg = panel_ditem_editor_new (NULL, NULL, NULL,
 							     _("Create Launcher"));
 				g_object_set_data_full (G_OBJECT (dlg), "dir",
-							g_strdup (uri),
+							g_strdup (path),
 							(GDestroyNotify)g_free);
 
 				panel_ditem_register_save_uri_func (PANEL_DITEM_EDITOR (dlg),
@@ -156,9 +164,7 @@ main (int argc, char * argv[])
 				 * secondly we will soon exit */
 			        desktops[i] = g_build_path ("/", uri,
 							    ".directory", NULL);
-				g_free (uri);
 				i--;
-				continue;
 			} else if (type == G_FILE_TYPE_REGULAR
 				   && g_str_has_suffix (desktops [i],
 					   		".directory")
@@ -213,12 +219,11 @@ main (int argc, char * argv[])
 		}
 
 		g_free (uri);
+		g_free (path);
 	}
 
 	if (dialogs > 0)
 		gtk_main ();
 
-	g_object_unref (program);
-
         return 0;
 }
