@@ -447,6 +447,7 @@ on_factory_appeared (GDBusConnection   *connection,
 
 static void
 panel_applet_container_get_applet (PanelAppletContainer *container,
+				   GdkScreen            *screen,
 				   const gchar          *iid,
 				   GVariant             *props,
 				   GCancellable         *cancellable,
@@ -455,7 +456,7 @@ panel_applet_container_get_applet (PanelAppletContainer *container,
 {
 	GSimpleAsyncResult *result;
 	AppletFactoryData  *data;
-	gint                screen;
+	gint                screen_number;
 	gchar              *bus_name;
 	gchar              *factory_id;
 	gchar              *applet_id;
@@ -480,12 +481,14 @@ panel_applet_container_get_applet (PanelAppletContainer *container,
 	factory_id = g_strndup (iid, strlen (iid) - strlen (applet_id));
 	applet_id += 2;
 
-	screen = gdk_screen_get_number (gtk_widget_get_screen (container->priv->socket));
+	/* we can't use the screen of the container widget since it's not in a
+	 * widget hierarchy yet */
+	screen_number = gdk_screen_get_number (screen);
 
 	data = g_new (AppletFactoryData, 1);
 	data->result = result;
 	data->factory_id = factory_id;
-	data->parameters = g_variant_new ("(si*)", applet_id, screen, props);
+	data->parameters = g_variant_new ("(si*)", applet_id, screen_number, props);
 	data->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
 
 	bus_name = g_strdup_printf (PANEL_APPLET_BUS_NAME, factory_id);
@@ -504,6 +507,7 @@ panel_applet_container_get_applet (PanelAppletContainer *container,
 
 void
 panel_applet_container_add (PanelAppletContainer *container,
+			    GdkScreen            *screen,
 			    const gchar          *iid,
 			    GCancellable         *cancellable,
 			    GAsyncReadyCallback   callback,
@@ -515,7 +519,7 @@ panel_applet_container_add (PanelAppletContainer *container,
 
 	panel_applet_container_cancel_pending_operations (container);
 
-	panel_applet_container_get_applet (container, iid, properties,
+	panel_applet_container_get_applet (container, screen, iid, properties,
 					   cancellable, callback, user_data);
 }
 
@@ -794,3 +798,47 @@ panel_applet_container_child_popup_menu_finish (PanelAppletContainer *container,
 
 	return !g_simple_async_result_propagate_error (simple, error);
 }
+
+void
+panel_applet_container_child_popup_edit_menu (PanelAppletContainer *container,
+					      guint                 button,
+					      guint32               timestamp,
+					      GCancellable         *cancellable,
+					      GAsyncReadyCallback   callback,
+					      gpointer              user_data)
+{
+	GSimpleAsyncResult *result;
+	GDBusProxy         *proxy = container->priv->applet_proxy;
+
+	if (!proxy)
+		return;
+
+	result = g_simple_async_result_new (G_OBJECT (container),
+					    callback,
+					    user_data,
+					    panel_applet_container_child_popup_edit_menu);
+
+	g_dbus_connection_call (g_dbus_proxy_get_connection (proxy),
+				g_dbus_proxy_get_name (proxy),
+				g_dbus_proxy_get_object_path (proxy),
+				PANEL_APPLET_INTERFACE,
+				"PopupEditMenu",
+				g_variant_new ("(uu)", button, timestamp),
+				NULL,
+				G_DBUS_CALL_FLAGS_NO_AUTO_START,
+				-1, cancellable,
+				child_popup_menu_cb,
+				result);
+}
+
+gboolean
+panel_applet_container_child_popup_edit_menu_finish (PanelAppletContainer *container,
+						     GAsyncResult         *result,
+						     GError              **error)
+{
+	GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (result);
+
+	g_warn_if_fail (g_simple_async_result_get_source_tag (simple) == panel_applet_container_child_popup_edit_menu);
+
+	return !g_simple_async_result_propagate_error (simple, error);
+}
