@@ -34,6 +34,7 @@
 #include <gdk/gdkx.h>
 
 #include "panel-applets-manager.h"
+#include "panel-bindings.h"
 #include "panel-profile.h"
 #include "panel.h"
 #include "applet.h"
@@ -90,58 +91,34 @@ struct _PanelAppletFramePrivate {
 	guint            has_handle : 1;
 };
 
-static void
-panel_applet_frame_paint (GtkWidget    *widget,
-			  GdkRectangle *area)
+static gboolean
+panel_applet_frame_draw (GtkWidget *widget,
+                         cairo_t   *cr)
 {
-	PanelAppletFrame *frame;
+        PanelAppletFrame *frame = PANEL_APPLET_FRAME (widget);
+	GtkStyleContext *context;
 
-	frame = PANEL_APPLET_FRAME (widget);
+        if (GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->draw)
+                GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->draw (widget, cr);
 
 	if (!frame->priv->has_handle)
-		return;
-  
-	if (gtk_widget_is_drawable (widget)) {
-		GtkOrientation orientation = GTK_ORIENTATION_HORIZONTAL;
+		return FALSE;
 
-		switch (frame->priv->orientation) {
-		case PANEL_ORIENTATION_TOP:
-		case PANEL_ORIENTATION_BOTTOM:
-			orientation = GTK_ORIENTATION_VERTICAL;
-			break;
-		case PANEL_ORIENTATION_LEFT:
-		case PANEL_ORIENTATION_RIGHT:
-			orientation = GTK_ORIENTATION_HORIZONTAL;
-			break;
-		default:
-			g_assert_not_reached ();
-			break;
-		}
+	context = gtk_widget_get_style_context (widget);
+	gtk_style_context_save (context);
+	gtk_style_context_set_state (context, gtk_widget_get_state_flags (widget));
 
-		gtk_paint_handle (
-			widget->style, widget->window,
-			GTK_WIDGET_STATE (widget),
-			GTK_SHADOW_OUT,
-			area, widget, "handlebox",
-			frame->priv->handle_rect.x,
-                        frame->priv->handle_rect.y,
-                        frame->priv->handle_rect.width,
-                        frame->priv->handle_rect.height,
-                        orientation);
-	}
-}
+	cairo_save (cr);
+        gtk_render_handle (context, cr,
+			   frame->priv->handle_rect.x,
+			   frame->priv->handle_rect.y,
+			   frame->priv->handle_rect.width,
+			   frame->priv->handle_rect.height);
+	cairo_restore (cr);
 
-static gboolean
-panel_applet_frame_expose (GtkWidget      *widget,
-			   GdkEventExpose *event)
-{
-	if (gtk_widget_is_drawable (widget)) {
-		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->expose_event (widget, event);
+	gtk_style_context_restore (context);
 
-		panel_applet_frame_paint (widget, &event->area);
-	}
-
-	return FALSE;
+        return FALSE;
 }
 
 static void
@@ -167,39 +144,76 @@ panel_applet_frame_update_background_size (PanelAppletFrame *frame,
 }
 
 static void
-panel_applet_frame_size_request (GtkWidget      *widget,
-				 GtkRequisition *requisition)
+panel_applet_frame_get_preferred_width(GtkWidget *widget, gint *minimal_width, gint *natural_width)
 {
 	PanelAppletFrame *frame;
 	GtkBin           *bin;
-	GtkRequisition    child_requisition;  
+	GtkWidget        *child;
+	guint             border_width;
 
 	frame = PANEL_APPLET_FRAME (widget);
 	bin = GTK_BIN (widget);
 
 	if (!frame->priv->has_handle) {
-		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->size_request (widget, requisition);
+		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->get_preferred_width (widget, minimal_width, natural_width);
 		return;
 	}
 
-	if (bin->child && gtk_widget_get_visible (bin->child)) {
-		gtk_widget_size_request (bin->child, &child_requisition);
+	child = gtk_bin_get_child (bin);
+	if (child && gtk_widget_get_visible (child))
+		gtk_widget_get_preferred_width (child, minimal_width, natural_width);
 
-		requisition->width  = child_requisition.width;
-		requisition->height = child_requisition.height;
-	}
-
-	requisition->width += GTK_CONTAINER (widget)->border_width;
-	requisition->height += GTK_CONTAINER (widget)->border_width;
+	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+	*minimal_width += border_width;
+	*natural_width += border_width;
 
 	switch (frame->priv->orientation) {
 	case PANEL_ORIENTATION_TOP:
 	case PANEL_ORIENTATION_BOTTOM:
-		requisition->width += HANDLE_SIZE;
+		*minimal_width += HANDLE_SIZE;
+		*natural_width += HANDLE_SIZE;
 		break;
 	case PANEL_ORIENTATION_LEFT:
 	case PANEL_ORIENTATION_RIGHT:
-		requisition->height += HANDLE_SIZE;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
+
+static void
+panel_applet_frame_get_preferred_height(GtkWidget *widget, gint *minimal_height, gint *natural_height)
+{
+	PanelAppletFrame *frame;
+	GtkBin           *bin;
+	GtkWidget        *child;
+	guint             border_width;
+
+	frame = PANEL_APPLET_FRAME (widget);
+	bin = GTK_BIN (widget);
+
+	if (!frame->priv->has_handle) {
+		GTK_WIDGET_CLASS (panel_applet_frame_parent_class)->get_preferred_height (widget, minimal_height, natural_height);
+		return;
+	}
+
+	child = gtk_bin_get_child (bin);
+	if (child && gtk_widget_get_visible (child))
+		gtk_widget_get_preferred_height (child, minimal_height, natural_height);
+
+	border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+	*minimal_height += border_width;
+	*natural_height += border_width;
+
+	switch (frame->priv->orientation) {
+	case PANEL_ORIENTATION_LEFT:
+	case PANEL_ORIENTATION_RIGHT:
+		*minimal_height += HANDLE_SIZE;
+		*natural_height += HANDLE_SIZE;
+		break;
+	case PANEL_ORIENTATION_TOP:
+	case PANEL_ORIENTATION_BOTTOM:
 		break;
 	default:
 		g_assert_not_reached ();
@@ -213,13 +227,18 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 {
 	PanelAppletFrame *frame;
 	GtkBin           *bin;
+	GtkWidget        *child;
+	GdkWindow        *window;
 	GtkAllocation     new_allocation;
 	GtkAllocation     old_allocation;
+	GtkAllocation     widget_allocation;
 
-	old_allocation.x      = widget->allocation.x;
-	old_allocation.y      = widget->allocation.y;
-	old_allocation.width  = widget->allocation.width;
-	old_allocation.height = widget->allocation.height;
+	gtk_widget_get_allocation (widget, &widget_allocation);
+
+	old_allocation.x      = widget_allocation.x;
+	old_allocation.y      = widget_allocation.y;
+	old_allocation.width  = widget_allocation.width;
+	old_allocation.height = widget_allocation.height;
 
 	frame = PANEL_APPLET_FRAME (widget);
 	bin = GTK_BIN (widget);
@@ -233,7 +252,9 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 		return;
 	}
 
-	widget->allocation = *allocation;
+	window = gtk_widget_get_window (widget);
+	child = gtk_bin_get_child (bin);
+	gtk_widget_set_allocation (widget, allocation);
 
 	frame->priv->handle_rect.x = 0;
 	frame->priv->handle_rect.y = 0;
@@ -283,18 +304,21 @@ panel_applet_frame_size_allocate (GtkWidget     *widget,
 	     new_allocation.y != frame->priv->child_allocation.y ||
 	     new_allocation.width != frame->priv->child_allocation.width ||
 	     new_allocation.height != frame->priv->child_allocation.height))
-	 	gdk_window_invalidate_rect (widget->window, &widget->allocation, FALSE);
+		gdk_window_invalidate_rect (window, &widget_allocation, FALSE);
 
 	if (gtk_widget_get_realized (widget)) {
-		gdk_window_move_resize (widget->window,
-			allocation->x + GTK_CONTAINER (widget)->border_width,
-			allocation->y + GTK_CONTAINER (widget)->border_width,
-			MAX (allocation->width - GTK_CONTAINER (widget)->border_width * 2, 0),
-			MAX (allocation->height - GTK_CONTAINER (widget)->border_width * 2, 0));
+		guint border_width;
+
+		border_width = gtk_container_get_border_width (GTK_CONTAINER (widget));
+		gdk_window_move_resize (window,
+			allocation->x + border_width,
+			allocation->y + border_width,
+			MAX (allocation->width - border_width * 2, 0),
+			MAX (allocation->height - border_width * 2, 0));
 	}
 
-	if (bin->child && gtk_widget_get_visible (bin->child))
-		gtk_widget_size_allocate (bin->child, &new_allocation);
+	if (child && gtk_widget_get_visible (child))
+		gtk_widget_size_allocate (child, &new_allocation);
   
 	frame->priv->child_allocation = new_allocation;
 
@@ -322,13 +346,16 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 {
 	PanelAppletFrame *frame;
 	gboolean          handled = FALSE;
+	GdkDisplay       *display;
+	GdkDevice        *pointer;
+	GdkDeviceManager *device_manager;
 
 	frame = PANEL_APPLET_FRAME (widget);
 
 	if (!frame->priv->has_handle)
 		return handled;
 
-	if (event->window != widget->window)
+	if (event->window != gtk_widget_get_window (widget))
 		return FALSE;
 
 	switch (event->button) {
@@ -350,11 +377,23 @@ panel_applet_frame_button_changed (GtkWidget      *widget,
 	case 3:
 		if (event->type == GDK_BUTTON_PRESS ||
 		    event->type == GDK_2BUTTON_PRESS) {
-			gdk_pointer_ungrab (GDK_CURRENT_TIME);
+			guint modifiers;
 
-			PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_menu (frame,
-									  event->button,
-									  event->time);
+			modifiers = event->state & GDK_MODIFIER_MASK;
+
+			display = gtk_widget_get_display (widget);
+			device_manager = gdk_display_get_device_manager (display);
+			pointer = gdk_device_manager_get_client_pointer (device_manager);
+			gdk_device_ungrab (pointer, GDK_CURRENT_TIME);
+
+			if (modifiers == panel_bindings_get_mouse_button_modifier_keymask ())
+				PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_edit_menu (frame,
+										       event->button,
+										       event->time);
+			else
+				PANEL_APPLET_FRAME_GET_CLASS (frame)->popup_menu (frame,
+										  event->button,
+										  event->time);
 
 			handled = TRUE;
 		} else if (event->type == GDK_BUTTON_RELEASE)
@@ -391,8 +430,9 @@ panel_applet_frame_class_init (PanelAppletFrameClass *klass)
 
 	gobject_class->finalize = panel_applet_frame_finalize;
 
-	widget_class->expose_event         = panel_applet_frame_expose;
-	widget_class->size_request         = panel_applet_frame_size_request;
+	widget_class->draw                 = panel_applet_frame_draw;
+	widget_class->get_preferred_width  = panel_applet_frame_get_preferred_width;
+	widget_class->get_preferred_height = panel_applet_frame_get_preferred_height;
 	widget_class->size_allocate        = panel_applet_frame_size_allocate;
 	widget_class->button_press_event   = panel_applet_frame_button_changed;
 	widget_class->button_release_event = panel_applet_frame_button_changed;
@@ -427,7 +467,7 @@ panel_applet_frame_sync_menu_state (PanelAppletFrame *frame)
 	gboolean     movable;
 	gboolean     removable;
 
-	panel_widget = PANEL_WIDGET (GTK_WIDGET (frame)->parent);
+	panel_widget = PANEL_WIDGET (gtk_widget_get_parent (GTK_WIDGET (frame)));
 
 	movable = panel_applet_can_freely_move (frame->priv->applet_info);
 	removable = panel_profile_id_lists_are_writable ();
@@ -461,13 +501,18 @@ void
 panel_applet_frame_change_background (PanelAppletFrame    *frame,
 				      PanelBackgroundType  type)
 {
+	GtkWidget *parent;
+
 	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
-	g_return_if_fail (PANEL_IS_WIDGET (GTK_WIDGET (frame)->parent));
+
+	parent = gtk_widget_get_parent (GTK_WIDGET (frame));
+
+	g_return_if_fail (PANEL_IS_WIDGET (parent));
 
 	if (frame->priv->has_handle) {
 		PanelBackground *background;
 
-		background = &PANEL_WIDGET (GTK_WIDGET (frame)->parent)->background;
+		background = &PANEL_WIDGET (parent)->background;
 		panel_background_change_background_on_widget (background,
 							      GTK_WIDGET (frame));
 	}
@@ -589,11 +634,14 @@ _panel_applet_frame_get_background_string (PanelAppletFrame    *frame,
 					   PanelWidget         *panel,
 					   PanelBackgroundType  type)
 {
+	GtkAllocation allocation;
 	int x;
 	int y;
 
-	x = GTK_WIDGET (frame)->allocation.x;
-	y = GTK_WIDGET (frame)->allocation.y;
+	gtk_widget_get_allocation (GTK_WIDGET (frame), &allocation);
+
+	x = allocation.x;
+	y = allocation.y;
 
 	if (frame->priv->has_handle) {
 		switch (frame->priv->orientation) {
@@ -744,11 +792,12 @@ void
 _panel_applet_frame_applet_move (PanelAppletFrame *frame)
 {
 	GtkWidget *widget = GTK_WIDGET (frame);
+	GtkWidget *parent = gtk_widget_get_parent (widget);
 
-	if (!PANEL_IS_WIDGET (widget->parent))
+	if (!PANEL_IS_WIDGET (parent))
 		return;
 
-	panel_widget_applet_drag_start (PANEL_WIDGET (widget->parent),
+	panel_widget_applet_drag_start (PANEL_WIDGET (parent),
 					widget,
 					PW_DRAG_OFF_CENTER,
 					GDK_CURRENT_TIME);
@@ -758,7 +807,7 @@ void
 _panel_applet_frame_applet_lock (PanelAppletFrame *frame,
 				 gboolean          locked)
 {
-	PanelWidget *panel_widget = PANEL_WIDGET (GTK_WIDGET (frame)->parent);
+	PanelWidget *panel_widget = PANEL_WIDGET (gtk_widget_get_parent (GTK_WIDGET (frame)));
 
 	if (panel_widget_get_applet_locked (panel_widget, GTK_WIDGET (frame)) == locked)
 		return;
@@ -782,6 +831,12 @@ panel_applet_frame_activating_free (PanelAppletFrameActivating *frame_act)
 	g_slice_free (PanelAppletFrameActivating, frame_act);
 }
 
+GdkScreen *
+panel_applet_frame_activating_get_screen (PanelAppletFrameActivating *frame_act)
+{
+	return gtk_widget_get_screen (GTK_WIDGET (frame_act->panel));
+}
+
 PanelOrientation
 panel_applet_frame_activating_get_orientation (PanelAppletFrameActivating *frame_act)
 {
@@ -913,11 +968,15 @@ panel_applet_frame_load (const gchar *iid,
 	g_return_if_fail (id != NULL);
 
 	if (g_slist_find_custom (no_reload_applets, id,
-				 (GCompareFunc) strcmp))
+				 (GCompareFunc) strcmp)) {
+		panel_applet_stop_loading (id);
 		return;
+	}
 
-	if (panel_lockdown_is_applet_disabled (iid))
+	if (panel_lockdown_is_applet_disabled (iid)) {
+		panel_applet_stop_loading (id);
 		return;
+	}
 
 	frame_act = g_slice_new0 (PanelAppletFrameActivating);
 	frame_act->locked   = locked;
@@ -943,9 +1002,11 @@ panel_applet_frame_load_from_gconf (PanelWidget *panel_widget,
 	g_return_if_fail (panel_widget != NULL);
 	g_return_if_fail (id != NULL);
 
-	applet_iid = panel_compatiblity_get_applet_iid (id);
-	if (!applet_iid)
+	applet_iid = panel_compatibility_get_applet_iid (id);
+	if (!applet_iid) {
+		panel_applet_stop_loading (id);
 		return;
+	}
 
 	panel_applet_frame_load (applet_iid, panel_widget,
 				 locked, position, TRUE, id);
