@@ -25,7 +25,6 @@ void            panel_lock_screen_action           (GdkScreen    *screen,
                                                     const char   *action);
 gboolean        panel_lock_screen_action_available (const char   *action);
 
-char *          panel_util_icon_remove_extension (const char *icon);
 char *          panel_find_icon         (GtkIconTheme  *icon_theme,
 					 const char    *icon_name,
 					 int            size);
@@ -64,6 +63,14 @@ void panel_util_set_tooltip_text (GtkWidget  *widget,
 
 GFile *panel_util_get_file_optional_homedir (const char *location);
 
+void panel_util_key_event_is_popup (GdkEventKey *event,
+				    gboolean    *is_popup,
+				    gboolean    *is_popup_modifier);
+
+void panel_util_key_event_is_popup_panel (GdkEventKey *event,
+					  gboolean    *is_popup,
+					  gboolean    *is_popup_modifier);
+
 G_END_DECLS
 
 #endif /* PANEL_UTIL_H */
